//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Boards 1.1.1
// Board : Seeed nRF52 Boards / Seeed XIAO nRF52840
// important : need for Serial.print() <Arduino.h> <Adafruit_TinyUSB.h>
// \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
//----------------------------------------------------------------------------------------------

// 2023/07/07
// Central sketch : nRF52_XIAO_LightSleepAdvScan_peripheral.ino

#include <Arduino.h>
#include <Adafruit_TinyUSB.h>     // For Serial.print()
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "Fonts/FreeSans9pt7b.h"
#include "Fonts/FreeSerif9pt7b.h"
#include <bluefruit.h>            // \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src

#define LOOP_TIME    1            // Loop time [sec]

#define dataNum 12                // CID 2, padding data 2, mS data 4, uS data 4 (max 26)
union unionData {                 // Union for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

const char* versionNumber = "0.99";   // Central sketch version number
uint32_t mS, uS;

// Instances of the class
Adafruit_SSD1306 display(128, 64, &Wire, -1);


void setup()
{
  // Serial port initialization
  Serial.begin(115200);
  delay(2000);

  // LED intialization
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);

  // Initialization of display SSD1306
  Wire.begin();
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))
  {
    Serial.println("Initialization failure of SSD1306");
  }
  display.clearDisplay();
  display.setFont(&FreeSans9pt7b);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 15);
  display.println("Light Sleep");
  display.setCursor(0, 31);
  display.println("BLE Central");
  display.setCursor(0, 47);
  display.print("Ver   : ");
  display.println(versionNumber);
  display.display();
  delay(5000);
  display.clearDisplay();

  // Initialization of Bruefruit
  Bluefruit.configCentralBandwidth(BANDWIDTH_MAX);  
  Bluefruit.begin(1, 1);
  Bluefruit.setName("XIAO BLE Central");

  // Blue LED blinking interval setting
  Bluefruit.setConnLedInterval(100);

  // Scanner settings
  Bluefruit.Scanner.setRxCallback(scan_callback);
  Bluefruit.Scanner.setIntervalMS(50, 25);    // interval:50mS  window:25mS  
  Bluefruit.Scanner.filterMSD(0x0059);        // Manufacturer Specific Data, CID nordic  
  Bluefruit.Scanner.useActiveScan(false);
  Bluefruit.Scanner.start(0);                 // Continuous Scanning

  Serial.println("End of initialization");   
}

//******************************************************************************************
void loop()
{
  uint32_t timestamp = millis();

  digitalWrite(LED_GREEN, LOW);
 
  // Restore and display incoming data     
  mS = ud.dataBuff32[1];
  uS = ud.dataBuff32[2];
    
  Serial.print(ud.dataBuff32[1]);
  Serial.print(" ");
  Serial.print(ud.dataBuff32[2]);
  Serial.println(" ");

  display.clearDisplay();
  display.setCursor(0, 15);
  display.print("mS "); display.print(mS);
  display.setCursor(0, 31);
  display.print("uS "); display.print(uS);
  display.setCursor(0, 47);
  display.print("minute "); display.print(mS/60000.0, 3);  
  display.display();
 
  digitalWrite(LED_GREEN, HIGH);

  while(millis() - timestamp < 1000 * LOOP_TIME);
}

//*****************************************************************************************************
// Callback invoked when scanner pick up an advertising data
void scan_callback(ble_gap_evt_adv_report_t* report)
{
  digitalWrite(LED_RED, LOW);
  
//  Bluefruit.Central.connect(report);
  Serial.println("Timestamp Addr              Rssi Data");

  Serial.printf("%09d ", millis());
  
  // MAC is in little endian --> print reverse
  Serial.printBufferReverse(report->peer_addr.addr, 6, ':');
  Serial.print(" ");

  Serial.print(report->rssi);
  Serial.print("  ");

  Serial.printBuffer(report->data.p_data, report->data.len, '-');
  Serial.println();
  Serial.print("Manufacturer Specific Data : ");
  for(int i = 0; i < dataNum; i++) {
    ud.dataBuff8[i] = report->data.p_data[i +5];
    Serial.print(ud.dataBuff8[i], HEX); Serial.print(" ");
  }
  Serial.println();

  // For Softdevice v6: after received a report, scanner will be paused
  // We need to call Scanner resume() to continue scanning
  Bluefruit.Scanner.resume();

  digitalWrite(LED_RED, HIGH);
}
