//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.1
// Board : Seeed nRF52 Boards / Seeed XIAO nRF52840 Sense
// important : need for Serial.print() <Arduino.h> <Adafruit_TinyUSB.h>
// \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
//----------------------------------------------------------------------------------------------

// 2023/07/07
// Central sketch : nRF52_XIAO_LightSleepAdvScan_central.ino
// Need to be deleted /Documents/Arduino/libraries/SdFat

#include <Arduino.h>
#include <Adafruit_TinyUSB.h>   // For Serial.print()
#include <bluefruit.h>          // \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\src
#include <Adafruit_SPIFlash.h>  // Need to be deleted /Documents/Arduino/libraries/SdFat

#define SLEEP_TIME 10   // Sleep time [sec]
#define dataNum 12      // max 26 bytes CID2+payload24
union unionData {       // Union for data type conversion
  uint32_t dataBuff32[dataNum / 4];
  uint16_t dataBuff16[dataNum / 2];
  uint8_t dataBuff8[dataNum];
};
union unionData ud;
uint32_t mS, uS;

Adafruit_FlashTransport_QSPI flashTransport;

void setup() {
  // Serial port initialization
  Serial.begin(115200);
  delay(2000);

  // LED initialization
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);

  // Enable DC-DC converter
  NRF_POWER->DCDCEN = 1;  // Enable DC/DC converter for REG1 stage

  // Sleep QSPI flash
  flashTransport.begin();
  flashTransport.runCommand(0xB9);  // enter deep power-down mode
  flashTransport.end();

  // Initialization of Bruefruit class
  Bluefruit.begin();
  Bluefruit.setTxPower(4);  // Check bluefruit.h:151 for supported values
  Bluefruit.autoConnLed(false); // turn off LED to save power   Sleep Current 4uA
//  Bluefruit.setConnLedInterval(50); // Should not be used, Sleep current 300uA

  Serial.println("End of initialization");
}

//*************************************************************************************************************
void loop() {
  digitalWrite(LED_GREEN, LOW);
    
/*
See \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.1\libraries\Bluefruit52Lib\examples\Projects\rssi_proximity\
rssi_proximity_peripheral.ino:99
  Alternative Solution: Manufacturer Specific Data (MSD)
  You could also send a custom MSD payload and filter for the 'Company ID'
  via 'Bluefruit.Scanner.filterMSD(CID);', although this does require a
  valid CID, which is why the UUID method above is more appropriate in
  most situations. For a complete list of valid company IDs see:
  https://www.bluetooth.com/specifications/assigned-numbers/company-identifiers
  For test purposes, 0xFFFF CAN be used, but according to the Bluetooth SIG:
  > "This value may be used in the internal and interoperability tests before a
  >  Company ID has been assigned. This value shall not be used in shipping end
  >  products."
  uint8_t msd_payload[4]; // Two bytes are required for the CID, so we have 2 bytes user data, expand as needed
  uint16_t msd_cid = 0xFFFF;
  memset(msd_payload, 0, sizeof(msd_payload));
  memcpy(msd_payload, (uint8_t*)&msd_cid, sizeof(msd_cid));
  msd_payload[2] = 0x11;
  msd_payload[3] = 0x22;
  Bluefruit.Advertising.addData(BLE_GAP_AD_TYPE_MANUFACTURER_SPECIFIC_DATA, msd_payload, sizeof(msd_payload));
*/
  // Manufacturer Specific Data setting
  mS = millis();
  uS = micros();
  ud.dataBuff16[0] = 0x0059;  // nordic CID
  ud.dataBuff16[1] = 0x0000;  // padding  
  ud.dataBuff32[1] = mS;      // payload
  ud.dataBuff32[2] = uS;

  Serial.print(mS);
  Serial.print(" ");
  Serial.println(uS);

  Bluefruit.Advertising.addFlags(BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE);
  Bluefruit.Advertising.addData(BLE_GAP_AD_TYPE_MANUFACTURER_SPECIFIC_DATA, ud.dataBuff8, dataNum);
  Bluefruit.Advertising.restartOnDisconnect(true);
  Bluefruit.Advertising.setIntervalMS(100, 100);  // fast mode 100mS, slow mode 100mS
  Bluefruit.Advertising.setFastTimeout(1);        // fast mode 1 sec
  Bluefruit.Advertising.start(1);                 // stop advertising after 1 sec, fast 1 sec slow 0 sec

  digitalWrite(LED_GREEN, HIGH);

  delay(SLEEP_TIME * 1000);           // System ON Sleep by FreeRTOS
  Bluefruit.Advertising.clearData();  // refresh advertised data for each cycle
}
